@@ -8,23 +8,24 @@ use feature qw(say state lexical_subs current_sub);
 no warnings qw(experimental::lexical_subs);
 
 package Spreadsheet::Edit::IO;
-our $VERSION = '3.012'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2023-05-27'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '3.013'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2023-05-30'; # DATE from Dist::Zilla::Plugin::OurDate
 
 # This module is derived from the old never-released Text:CSV::Spreadsheet
 
 use Exporter 'import';
-our @EXPORT_OK = qw(@sane_CSV_read_options @sane_CSV_write_options
-                    cx2let let2cx cxrx2sheetaddr convert_spreadsheet OpenAsCsv
-                    sheetname_from_spec filepath_from_spec
-                    form_spec_with_sheetname
-                   );
+
+our @EXPORT = qw/convert_spreadsheet OpenAsCsv cx2let let2cx cxrx2sheetaddr
+                 sheetname_from_spec filepath_from_spec 
+                 form_spec_with_sheetname/;
+
+our @EXPORT_OK = qw/@sane_CSV_read_options @sane_CSV_write_options/;
 
 # TODO: Provide "known_attributes" function ala Text::CSV::known_attributes()
 
 use version ();
 use Carp;
-sub oops(@) { @_=("\n".__PACKAGE__." oops:\n",@_,"\n"); goto &Carp::confess }
+sub oops(@) { @_=("\n".__PACKAGE__." **oops**: @_\n"); goto &Carp::confess }
 sub btw(@) { local $_=join("",@_); s/\n\z//s; warn((caller(0))[2].": $_\n"); }
 
 use File::Copy ();
@@ -41,18 +42,18 @@ use File::Basename qw(basename dirname);
 use File::Which qw/which/;
 use Guard qw(guard scope_guard);
 use Fcntl qw(:flock :seek);
-use List::Util qw/none all notall first/;
+use List::Util qw/none all notall first min max/;
 use Encode qw(encode decode);
 # DDI 5.015 is needed for 'qshlist'
-use Data::Dumper::Interp qw/vis visq dvis ivis avis qsh qshlist u/;
+use Data::Dumper::Interp qw/vis visq dvis dvisq ivis ivisq avis qsh qshlist u/;
 use File::Glob qw/bsd_glob GLOB_NOCASE/;
 use Digest::MD5 qw/md5_base64/;
+use Text::CSV ();
 
 use Spreadsheet::Edit::Log qw/log_call fmt_call log_methcall fmt_methcall/;
 our %SpreadsheetEdit_Log_Options = (
   is_public_api => sub{
-    # hide calls
-     $_[1][3] =~ /(?:::|^)[a-z][^:]*$/
+     $_[1][3] =~ /(?: ::|^ )(?: [a-z][^:]* | OpenAsCsv | ConvertSpreadsheet )$/x
   },
 );
 
@@ -219,7 +220,7 @@ sub _name2LOcharsetnum($) {
   local $_ = uc $enc;
   while (! $LO_charsets{$_}) {
     # successively remove - and other special characters
-    s/\W//a or croak "Unknown encoding name '$enc'";
+    s/\W//a or confess "LO charset: Unknown encoding name '$enc'";
   }
   $LO_charsets{$_}
 }
@@ -458,25 +459,58 @@ sub _create_tempdir_if_needed($) {
     my $pid = $$;
     my $euid = $>;
     my $user = getpwuid($euid) // $euid;
-    (my $dname = __PACKAGE__."_${user}_${pid}") =~ s/::/-/g;
+    (my $dname = __PACKAGE__."_${user}_${pid}_tempdir") =~ s/::/-/g;
     (my $path = path(File::Spec->tmpdir)->child($dname))->mkpath;
     $path
   };
 }
 END{ $tempdir->remove_tree if $tempdir; }
 
-# Compose {tempdir}/{ifbase}_<signature>_csvcach 
-sub _cachedir($) {
+# Compose a unique path under $tempdir.
+# This is *not* a "tempfile" or "tempdir" object which auto-destructs,
+# in fact it does not even exist yet and we don't know here which it will be.  
+# Either the user must remove it when they are done with it, or it will 
+# be removed when $tempdir is removed at process exit.
+#
+sub _path_under_tempdir($@) {
   my $opts = shift;
-  $opts->{inpath_fingerprint} 
-              //= _file_fingerprint($opts->{inpath_sans_sheet});
-  my $dname = $opts->{ifbase}."_".$opts->{inpath_fingerprint}."_csvcache";
-  $tempdir->child($dname)
+  my %args = (
+    words => [$opts->{ifbase}, $opts->{sheetname}],
+    @_
+  );
+  my $bname = join "_", grep{defined} @{$args{words}};
+  # Collisions occur when recursing to emulate Extract-by-name,
+  # or if the user repeatedly reads the same thing, etc.
+  state $seqnums = {};
+  if ($seqnums->{$bname}++) {
+    $bname .= "_".$seqnums->{$bname};  # append unique sequence number
+  }
+  $bname .= ".$args{suf}" if $args{suf};
+  return $tempdir->child($bname);
 }
 
-sub _convert_using_openlibre($$) {
-  my ($opts, $dst) = @_;
-  oops unless all{ $opts->{$_} } qw/inpath_sans_sheet cvt_from cvt_to/;
+# Compose csv cache subdir path
+sub _cachedir($) {
+  my $opts = shift;
+  _path_under_tempdir($opts,words => [$opts->{ifbase}, "csvcache"]);
+}
+
+## Copy an ephemeral temp file to a path under tempdir if needed
+#sub _make_file_permanent($$) {
+#  my ($opts, $path) = @_;
+#  if (eval{ $path->cached_temp }) { # didn't throw
+#    my $suf = $path->basename =~ /\.(\w+)$/a ? $1 : undef;
+#    my $newpath = _path_under_tempdir($opts, suf => $suf);
+#    $path->move($newpath);
+#    return $newpath
+#  } else {
+#    return $path
+#  }
+#}
+
+sub _convert_using_openlibre($$$) {
+  my ($opts, $src, $dst) = @_;
+  oops unless all{ $opts->{$_} } qw/cvt_from cvt_to/;
   oops if $opts->{allsheets} && ! _openlibre_supports_allsheets();
   oops if $opts->{sheetname} && ! _openlibre_supports_named_sheet();
   my $debug = $opts->{debug};
@@ -523,10 +557,9 @@ sub _convert_using_openlibre($$) {
       my $charset = _name2LOcharsetnum($enc); # dies if unknown enc
       my $colformats = "";
       if (my $cf = $opts->{col_formats}) {
-        #This mis-feature not tested, and probably will go away
-        $cf = [split /\//, $cf] if !ref($cf);  #  fmtA/fmtB/...
+        $cf = [split /[\/,]/, $cf] unless ref($cf);  #  fmtA/fmtB/...
         for (my $ix=0; $ix <= $#$cf; $ix++) {
-          local $_ = $cf->[$ix];
+          local $_ = $cf->[$ix] // 1;
              m#^([123459]|10)$#
           || s#^standard$#1#i
           || s#^text$#2#i
@@ -537,13 +570,15 @@ sub _convert_using_openlibre($$) {
           || s#^US.*English$#10#i
           || croak "Unknown format code '$_' in {col_formats}";
           $colformats .= "/" if $colformats;
-          $colformats .= "$ix/$_";
+          $colformats .= ($ix+1)."/$_";
         }
       }
       $filter_name.":"
       # Tokens 1-4: FldSep=',' TxtDelim='"' Charset FirstLineNum
-      # TODO: Detect separator and quote chacter from the data
-      . "44,34,$charset,1"
+      #. "44,34,$charset,1"
+      . ord($opts->{sep_char}//",").","
+      . ord($opts->{quote_char}//'"').","
+      . "$charset,1"
       # Token 5: Cell format codes:
       #  If variable-width cells (the norm): colnum/fmt/colnum/fmt...
       #    colnum: 1-based column number
@@ -555,9 +590,15 @@ sub _convert_using_openlibre($$) {
       . ",$colformats"
       # Token 6: MS-LCID Language Id; 0 or omitted means UI language
       . ","  # default: false
-      # Token 7: On input: "Quoted --> text". On output: "Quote text cells"
-      #   This must be false to recognize dates like "Jan 1, 2000" because
-      #   they by necessity must be quoted because of the embeded comma.
+      # Token 7: On input: true => Quoted cells are always read a 'text',
+      #  effectively disabling Token 8.  This must be false to recognize dates 
+      #   like "Jan 1, 2000" which by necessity must be quoted for the comma,
+      #   but will **CORRUPT** zip codes with leading zeroes!
+      
+#FIXME:
+#We need to identify *columns* which must be quoted and generate Token 5
+#accordingly.  With Tok7=false even "001" looses the leading zeroes!
+
       .",false" # default: false
       # Token 8: on input: "Detect Special Numbers", i.e. date or time values
       #   in human form, numbers in scientific (expondntial) notation etc.
@@ -587,7 +628,10 @@ sub _convert_using_openlibre($$) {
       my $charset = _name2LOcharsetnum($enc); # dies if unknown enc
       $filter_name.":"
       # Tokens 1-4: FldSep=, TxtDelim=" Charset FirstLineNum
-      ."44,34,$charset,1"
+      #."44,34,$charset,1"
+      . ord($opts->{sep_char}//",").","
+      . ord($opts->{quote_char}//'"').","
+      . "$charset,1"
       # Token 5: Cell format codes.  Only used for import? (see above)
       #   What about fixed-width?
       .","
@@ -596,9 +640,14 @@ sub _convert_using_openlibre($$) {
       .","
       # Token 7: QuoteAllTextCells
       # *** true will "quote" even single-bareword cells, which looks
-      # *** bad and makes t/ tests messier.  OTOH quotes provide information
-      # *** that such cells were not numbers or dates, etc.  
-      # *** So I'm now using "true" to always quote.
+      # *** bad and makes t/ tests messier, but preserves information
+      # *** that such cells were not numbers or dates, etc.  This ensures
+      # *** that Zip codes, etc. with leading zeroes won't be corrupted
+      # *** if converted back into a spreadsheet.
+      # ??? FIXME/TODO:
+      #    We could post-process to remove quotes from obviously-text fields.
+      #    Seems reasonable because we know the quoting convention used
+      #    (because we specify it here)... unlike pre-processing on input.
       .",true"
       # Token 8: on output: true to store number as numbers; false to
       #          store number cells as text.  No UI equivalent.
@@ -645,15 +694,18 @@ sub _convert_using_openlibre($$) {
   # will be deleted when $dirpath goes out of scope
   
   my @cmd = ($prog, "--headless", "--invisible",
+                    "--nolockcheck", "--norestore",
                     $ifilter ? ("--infilter=$ifilter") : (),
                     "--convert-to", 
                       $opts->{cvt_to}.($ofilter ? ":$ofilter" : ""),
                     "--outdir", $tdir,
-                    $opts->{inpath_sans_sheet});
+                    $src);
 
   $opts->{stdout_to_stderr} = 1;       # send "convert..." message to stderr
   $opts->{suppress_stderr} = !$debug;  # and suppress it unless tracing
 
+btw "############################## INPUT CONTENT $src :\n",vis(path($src)->slurp_utf8),"\n++++++++++++++++++++++++++++++++++++++++" if $debug;
+
   my $cmdstatus = _runcmd($opts, @cmd);
 
   if ($cmdstatus != 0) {
@@ -664,8 +716,8 @@ sub _convert_using_openlibre($$) {
   my @result_files = path($tdir)->children;
   btw dvis '>> @result_files' if $debug;
   if (@result_files == 0) {
-    croak $opts->{inpath_sans_sheet}." is missing or unreadable\n"
-      unless -r $opts->{inpath_sans_sheet};
+    croak qsh($src)." is missing or unreadable\n"
+      unless -r $src;
     croak "Something went wrong, ",path($prog)->basename," produced no output\n"
   }
 
@@ -702,10 +754,10 @@ sub _convert_using_openlibre($$) {
       File::Copy::move($result_files[0], $dst);
     }
   }
-}
+}#_convert_using_openlibre
 
-sub _convert_using_ssconvert($$) {
-  my ($opts, $dst) = @_;
+sub _convert_using_ssconvert($$$) {
+  my ($opts, $src, $dst) = @_;
   confess "Deprecated with extreme prejudice"; # no longer supported
 ##
 ##  foreach (qw/inpath cvt_to /)
@@ -850,7 +902,7 @@ sub form_spec_with_sheetname($$) {
   #$sheetname ? "${filepath}|||${sheetname}" : $filepath
 }
 
-our $default_input_encodings = "UTF-8,UTF-16BE,UTF-16LE,windows-1252";
+our $default_input_encodings = "UTF-8,windows-1252,UTF-16BE,UTF-16LE";
 our $default_output_encoding = "UTF-8";
 
 # Return digested %opts setting
@@ -928,10 +980,177 @@ sub _parse_iolayers($) {
   ($prefix, $enclist, $suffix);
 }
 
-# Detect cvt_to and input_encoding if a csv; 
-# Detect cvt_to and set default output_encoding if not specified
+# Detect cvt_from and cvt_to from filenames, or peeking at the data.
+# If input is CSV, detect encoding, separator and quote characters; 
+#   add quotes to values with leading zeroes (e.g. Zip codes) which would
+#   otherwise be corrupted by being read as numbers instead of text strings.
+#   The modified data is written to a temp file
+# Set default output_encoding if not specified
+# RETURNS: The effective input path, either inpath_sans_sheet or a tempfile
 sub _determine_enc_tofrom($) {
   my $opts = shift;
+  my $debug = $opts->{debug};
+  # Skip to ==BODY== below
+
+  my sub determine_input_encoding($) {
+    my $r2octets = shift;
+    # If user specified one encoding, use it; if user specified list, try them.
+    # If user did not specify, the default is a list to try.
+    $opts->{input_encoding} //= $default_input_encodings;
+    my @enclist = split m#,#, $opts->{input_encoding};
+    return 
+      if @enclist == 1; 
+    $$r2octets //= $opts->{inpath_sans_sheet}->slurp_raw;
+    for my $enc (@enclist) {
+      eval { decode($enc, $$r2octets, Encode::FB_CROAK|Encode::LEAVE_SRC) };
+      if ($@) {
+         btw "Input encoding '$enc' did not work...($@)\n" if $debug;
+         next;
+      }
+      btw "Input encoding '$enc' seems to work.\n" if $debug;
+      @enclist = ($enc);
+      last
+    }
+    #croak "Could not detect encoding of $opts->{inpath_sans_sheet}\n" 
+    confess "Could not detect encoding of $opts->{inpath_sans_sheet}\n" 
+      if @enclist > 1;
+    $opts->{input_encoding} = $enclist[0];
+  } #determine_input_encoding
+
+  my sub readparse_csv($@) {
+    my $fh = shift;
+    my %csvopts = (
+      @sane_CSV_read_options,
+      defined($opts->{quote_char}) ? (quote_char=>$opts->{quote_char}) : (),
+      defined($opts->{sep_char})   ? (sep_char=>$opts->{sep_char})     : (),
+      auto_diag => 2, # throw on error
+      @_
+    );
+    $csvopts{escape_char} = $csvopts{quote_char}; # must always be the same
+
+    my $csv = Text::CSV->new (\%csvopts)
+              or croak "Text::CSV->new: ", Text::CSV->error_diag(),
+                       dvis('\n## %csvopts\n');
+    my @rows;
+    while (my $F = $csv->getline( $fh )) {
+      push(@rows, $F);
+    }
+    \@rows
+  }
+
+  my sub open_input($) {
+    my $r2octets = shift;
+    oops unless $opts->{input_encoding};
+    my $fh;
+    my $pathish = defined($$r2octets) 
+                    ? \$$r2octets : $opts->{inpath_sans_sheet};
+    open($fh, "<:encoding($opts->{input_encoding})", $pathish) 
+      or die "$pathish : $!";
+    $fh
+  }
+
+  my sub determine_csv_q_sep($$) {
+    my ($r2octets, $r2rows) = @_;
+    return 
+      if defined($opts->{quote_char}) && defined($opts->{sep_char});
+
+    my $fh = open_input($r2octets);
+    
+#    my $chars;
+#    if (defined($$r2octets)
+#      $chars = decode($opts->{input_encoding},$$r2octets,Encode::FB_CROAK);
+#    }
+    
+    # Try combinations starting with the most-common '"' and ',' while
+    # parsing the file for unsafe unquoted values (throws on syntax error).
+    # The expectation is that the first try usually succeeds
+    Q: 
+    for my $q (defined($opts->{quote_char}) 
+                 ? ($opts->{quote_char}) : ("\"", "'")) {
+      my $found_q;
+      SEP: 
+      for my $sep (defined($opts->{sep_char}) 
+                     ? ($opts->{sep_char}) : (",","\t")) {
+        btw dvis '--- TRYING $q $sep ---' if $debug;
+#        # Preliminary check for an illegal use of the quote char
+#        if (defined($chars) 
+#            && $chars =~ /[^${q}${sep}\x{0D}\x{0A}]
+#                          ${q}
+#                          (?=[^${q}${sep}\x{0D}\x{0A}] | \z)/gx) {
+#          btw ivis '>>>quote_char CAN NOT BE $q with sep=$sep because q exists mid-field before pos ${\(pos($chars))}'
+#            if $debug;
+#          next SEP
+#        }
+        $$r2rows = eval{ readparse_csv($fh, quote_char=>$q, sep_char=>$sep) };
+        if ($@ eq "") {
+          warn ivis '>> Detected quote_char=$q sep_char=$sep\n' if $debug;
+          $opts->{quote_char} = $q;
+          $opts->{sep_char} = $sep;
+          last Q;
+        }
+        warn vis '$@\nq=$q sep=$sep did not work...\n' if $debug;
+        seek $fh, 0, SEEK_SET;
+      }
+    }
+    unless (defined($$r2rows)) {
+      confess "Input file is not valid CSV (or we have a bug)\n"
+    }
+  }#determine_csv_q_sep
+
+  my sub determine_csv_col_formats($$) {
+    my ($r2octets, $r2rows) = @_;
+    return 
+      if defined $opts->{col_formats};
+    $$r2rows //= do{
+      my $fh = open_input($r2octets);
+      readparse_csv($fh);
+    };
+    state $curr_yy = (localtime(time))[5];
+    my @col_formats;
+    for my $row (@{ $$r2rows }) {
+      for my $i (0..$#$row) {
+        for ($row->[$i]) {
+          # recognize obvious Y/M/D or M/D/Y or D/M/Y date forms
+          if (m#\b(?<y>(?:[12]\d)?\d\d)/(?<m>\d\d)/(?<d>\d\d)\b#) {
+            if ($+{d} > 12 && $+{d} <= 31 && $+{m} >= 1 && $+{m} <= 12 
+                 && ($+{y} < 100 || $+{y} >= 1000)) {
+              $col_formats[$i] = "YY/MM/DD";
+              btw "Recognized '$_' as $col_formats[$i] format" if $debug;
+              next
+            }
+          }
+          if (m#\b(?<m>\d\d)/(?<d>\d\d)/(?<y>(?:[12]\d)?\d\d)\b#) {
+            if ($+{y} < 100 || $+{y} >= 1000) {
+              if ($+{d} > 12 && $+{d} <= 31 && $+{m} >= 1 && $+{m} <= 12) {
+                $col_formats[$i] = "MM/DD/YY";
+                btw "Recognized '$_' as $col_formats[$i] format" if $debug;
+                next
+              }
+              elsif ($+{m} > 12 && $+{m} <= 31 && $+{d} >= 1 && $+{d} <= 12) {
+                $col_formats[$i] = "DD/MM/YY";
+                btw "Recognized '$_' as $col_formats[$i] format" if $debug;
+                next
+              }
+            }
+          }
+          # Things to force to be read as text fields:
+          # 1. Leading zeroes
+          # 2. Leading ascii minus (\x{2D}) rather than math minus \N{U+2212}.
+          #    This prevents conversion to the Unicode math minus when LO
+          #    reads the CSV.  The assumption is that if the input has an ascii
+          #    minus then the original spreadsheet format was "text" not
+          #    numeric.
+          if (/^[\x{2D}0]/) {
+            $col_formats[$i] = "text";
+            btw "Recognized '$_' as $col_formats[$i] format" if $debug;
+          }
+        }
+      }
+    }
+    $opts->{col_formats} = \@col_formats;
+  }#determine_csv_col_formats
+
+  # ==BODY==
   unless ($opts->{cvt_to}) {
     if ($opts->{outpath} && $opts->{outpath} =~ /\.([^.]+)$/) {
       $opts->{cvt_to} = $1;
@@ -941,55 +1160,48 @@ sub _determine_enc_tofrom($) {
       unless $opts->{cvt_to};
   }
   unless ($opts->{cvt_from}) {
-    if ($opts->{inpath_sans_sheet} 
-                          && $opts->{inpath_sans_sheet} =~ /\.([^.]+)$/) {
+    if ($opts->{inpath_sans_sheet} =~ /\.([^.]+)$/) {
       $opts->{cvt_from} = $1;
-      $opts->{cvt_from} =~ s/^\.txt$/.csv/i;
     } 
   }
-  # If we don't know what the input format is, or we do know it's CSV but
-  # don't know its encoding, examine the file content to figure it out.
+  $opts->{cvt_from} =~ s/^\.txt$/.csv/i if $opts->{cvt_from};
+
+  # Detect file format and, if CSV, encoding
+  my ($octets, $rows);
   if (!$opts->{cvt_from} || $opts->{cvt_from} eq "csv") {
-    my $octets;
-    $opts->{input_encoding} //= $default_input_encodings;
-    my @enclist = split m#,#, $opts->{input_encoding};
-    if (@enclist > 1) {
-      $octets = $opts->{inpath_sans_sheet}->slurp_raw;
-      for my $enc (@enclist) {
-        eval { decode($enc, $octets, Encode::FB_CROAK|Encode::LEAVE_SRC) };
-        if ($@) {
-           btw "Input encoding '$enc' did not work...($@)\n" if $opts->{debug};
-           next;
-        }
-        btw "Input encoding '$enc' seems to work.\n" if $opts->{debug};
-        @enclist = ($enc);
-        last
-      }
-      #croak "Could not detect encoding of $opts->{inpath_sans_sheet}\n" 
-      confess "Could not detect encoding of $opts->{inpath_sans_sheet}\n" 
-        if @enclist > 1;
-    }
-    $opts->{input_encoding} = $enclist[0];
-    if (!$opts->{cvt_from}) {
-      $octets //= $opts->{inpath_sans_sheet}->slurp_raw;
-      my $chars = decode($opts->{input_encoding}, $octets, Encode::FB_CROAK);
-      # Does it look like a csv file?
-      # This is not at all correct, e.g. doesn't know about "quoted" fields,
-      # but merely screens for total garbage
-      my $min_cols_minus1 = 3 - 1;
-      if ($chars =~ /\A(?:.*?,){$min_cols_minus1,}[^,]*[\x{0A}\x{0D}]/s
-          ||
-          $chars =~ /\A(?:.*?\t){$min_cols_minus1,}[^\t]*[\x{0A}\x{0D}]/s
-          ||
-          length($octets)==0) {
-        $opts->{cvt_from} = 'csv';
-      } else {
-        croak "Can not detect what kind of file ",qsh($opts->{inpath})," is\n";
+    determine_input_encoding(\$octets);
+  }
+  if (!$opts->{cvt_from}) {
+    # Detect the file format by looking at the data.  Actually, we only
+    # support CSV in this case, so this is just a (half-baked) sanity check.
+    eval {
+      determine_csv_q_sep(\$octets, \$rows);
+      if (!$opts->{cvt_from}) {
+        $rows //= do{
+          my $fh = open_input(\$octets);
+          readparse_csv($fh);
+        };
       }
+    };
+    if ($@ eq "") {
+      warn "> Detected $opts->{input_sans_csv} as a seemingly-valid CSV\n"
+        if $debug;
+      $opts->{cvt_from} = "csv";
+    } else {
+      croak "Can not detect what kind of file ",qsh($opts->{inpath})," is\n";
     }
   }
+
+  if ($opts->{cvt_from} eq "csv") {
+    determine_csv_col_formats(\$octets, \$rows);
+  } else {
+    oops if defined($octets) or defined($rows);
+  }
+
+  # Set default ouput_encoding if not specified
   $opts->{output_encoding} //= $default_output_encoding
     if $opts->{cvt_to} eq "csv";
+
 }#_determine_enc_tofrom
 
 sub _tool_extract_all_csvs($$) {
@@ -1001,10 +1213,10 @@ sub _tool_extract_all_csvs($$) {
   delete local $opts->{sheetname};
   local $opts->{allsheets} = 1;
   if (_openlibre_supports_allsheets()) {
-    _convert_using_openlibre($opts, $destdir);
+    _convert_using_openlibre($opts, $opts->{inpath_sans_sheet}, $destdir);
   }
   elsif (_ssconvert_supports_allsheets()) {
-    _convert_using_ssconvert($opts, $destdir);
+    _convert_using_ssconvert($opts, $opts->{inpath_sans_sheet}, $destdir);
   }
   else { croak "Can't extract 'allsheets'.  Please install LibreOffice 7.2 or newer" }
 }
@@ -1023,9 +1235,9 @@ sub _tool_extract_one_csv($$) {
 
   confess "should not get here" if $opts->{sheetname};
   if (_openlibre_features->{available}) {
-    _convert_using_openlibre($opts, $destpath);
+    _convert_using_openlibre($opts, $opts->{inpath_sans_sheet}, $destpath);
   } else {
-    _convert_using_ssconvert($opts, $destpath);
+    _convert_using_ssconvert($opts, $opts->{inpath_sans_sheet}, $destpath);
   }
 }
 sub _tool_can_extract_current_sheet() {
@@ -1039,10 +1251,14 @@ sub _tool_write_spreadsheet($$) {
   scope_guard { _release_lock($opts); };
 
   if (_openlibre_supports_writing($opts->{cvt_to})) {
-    _convert_using_openlibre($opts, $destpath);
+    if ($opts->{sheetname}) {
+      carp "WARNING: Sheet name when creating a spreadsheet will be ignored\n";
+      delete $opts->{sheetname};
+    }
+    _convert_using_openlibre($opts, $opts->{inpath_sans_sheet}, $destpath);
   }
   elsif (_ssconvert_supports_writing($opts->{cvt_to})) {
-    _convert_using_ssconvert($opts, $destpath);
+    _convert_using_ssconvert($opts, $opts->{inpath_sans_sheet}, $destpath);
   }
   else { croak "Can't create $opts->{cvt_to} spreadsheets.  Please install LibreOffice 7.2 or newer" }
 }
@@ -1051,7 +1267,7 @@ sub _tool_write_spreadsheet($$) {
 # Extract CSVs for every sheet into {outpath} (setting to tmpdir if not preset).
 # If cached CSVs are available they are moved into {outpath}/ .
 sub _extract_all_csvs($) {
-  my $opts = shift;
+  my ($opts) = @_;
   my $outpath = _final_outpath($opts);
   $outpath->mkpath; # nop if exists, croaks if conflicts with file
 
@@ -1062,7 +1278,7 @@ sub _extract_all_csvs($) {
 # Extract a single sheet into a CSV at {outpath} (defaulting to temp file).
 # If a cached CSV is available it is moved to {outpath}.
 sub _extract_one_csv($) {
-  my $opts = shift;
+  my ($opts) = @_;
   my $cachedirpath = _cachedir($opts);
 
   my sub _fill_csv_cache() {
@@ -1118,7 +1334,7 @@ sub _extract_one_csv($) {
   }
 }
 sub _write_spreadsheet($) {
-  my $opts = shift;
+  my ($opts) = @_;
 
   my $outpath = _final_outpath($opts);
   $outpath->remove unless -d $outpath;
@@ -1126,27 +1342,17 @@ sub _write_spreadsheet($) {
   _tool_write_spreadsheet($opts, $outpath);
 }
 
-# If {outpath} is not set, set it to a unique temporary path in $tempdir
-#
-# This is *not* a "tempfile" or "tempdir" object which auto-destructs,
-# in fact it does not even exist yet and we don't know here which it will be.  
-# Either the user must remove it when they are done with it, or it will 
-# be removed when $tempdir is removed a process exit.
-#
+# If {outpath} is not set, set it to a unique output path in $tempdir
 # Always returns {outpath} as a Path::Tiny object.
 sub _final_outpath($) {
   my $opts = shift;
-  $opts->{outpath} ? path($opts->{outpath}) : ($opts->{outpath} = do{
-     my $base = $opts->{ifbase}
-                .($opts->{sheetname} ? "_$opts->{sheetname}" : "");
-     # Collisions occur when recursing to emulate Extract-by-name,
-     # or if the user repeatedly reads the same exact thing
-     state %memory;
-     if ($memory{$base}++) {
-       $base .= $memory{$base};  # sequence number
-     }
-     $tempdir->child("${base}.$opts->{cvt_to}")
-  });
+  if (defined $opts->{outpath}) {
+    return path($opts->{outpath});
+  } else {
+    return( 
+      ($opts->{outpath}=_path_under_tempdir($opts, suf=>$opts->{cvt_to})) 
+    );
+  }
 }
 
 sub convert_spreadsheet(@) {
@@ -1157,7 +1363,10 @@ sub convert_spreadsheet(@) {
 
   _create_tempdir_if_needed(\%opts);
 
-  _determine_enc_tofrom(\%opts); # intuit cvt_from & cvt_to if possible
+  # intuit cvt_from & cvt_to, detect encoding, and pre-process .csv input
+  # if needed to avoid corruption of leading zeroes.
+  _determine_enc_tofrom(\%opts); 
+
   my $input_enc = $opts{input_encoding};
   my $output_enc = $opts{output_encoding};
 
@@ -1177,7 +1386,7 @@ sub convert_spreadsheet(@) {
     if (!$opts{allsheets}) {
       if ($input_enc ne $output_enc) {
         # Special case #1: in & out are CSVs but different encodings.
-        warn "> Transcoding csv $opts{inpath_sans_sheet}:  $input_enc -> $output_enc\n" 
+        warn "> Transcoding csv:  $input_enc -> $output_enc\n" 
           if $opts{debug};
         my $octets = $opts{inpath_sans_sheet}->slurp_raw;
         my $chars = decode($input_enc, $octets, Encode::FB_CROAK);
@@ -1203,13 +1412,14 @@ sub convert_spreadsheet(@) {
     else {
       # Special case #2: <allsheets> with input already a csv:
       #   Leave a symlink to the input in the <outpath> directory.
-      croak "transcoding not impl in this situation"
+      croak "transcoding not implemented in this situation"
         if ($input_enc ne $output_enc);
       my $outpath = path(_final_outpath(\%opts));
       $outpath->mkpath; # nop if exists, croaks if conflicts with file
       my $dest = $outpath->child( $opts{ifbase}.".csv" );
-      symlink($opts{inpath_sans_sheet}, $dest)
-        or croak "symlink $opts{inpath_sans_sheet} <-- $dest : $!";
+      my $inpath = $opts{inpath_sans_sheet};
+      symlink($inpath, $dest)
+        or croak "symlink $inpath <-- $dest : $!";
       warn "  No conversion needed! Leaving symlink at ", qsh($dest),"\n"
         if $opts{verbose};
       $done = 1;
@@ -1242,8 +1452,8 @@ sub convert_spreadsheet(@) {
 
 # Open as a CSV, intuiting input encoding, converting spreadsheet if necessary.
 #
-# The :crlf will be used, which translates DOS CR,LF to \n while passing
-# *nix bare LF through unmolested.
+# :crlf translation is enabled on the resulting file handle, which converts 
+# DOS CR,LF to \n while passing *nix bare LF through unmolested.
 #
 # Input argument(s) are the same as for convert_spreadsheet (except
 # outpath may not be specified).
@@ -1255,8 +1465,6 @@ sub OpenAsCsv {
               cvt_to => 'csv',
             );
   # TODO: Rename {path} to {inpath} in all usages and rm this cruft;
-  # TODO: Consider renaming {inpath} as {input} bc it can be a filehandle
-  #   (massive changes required; in some contexts it must be a path...)
   carp "Obsolete OpenAsCsv usage: Change path to inpath\n"
     if exists($opts{path}) and !$opts{silent};
   $opts{inpath} //= delete $opts{path}; # be compatible with old API 
@@ -1268,7 +1476,7 @@ sub OpenAsCsv {
   my $h = convert_spreadsheet($inpath, %opts, verbose => $opts{debug});
   oops "sheetname key bug" if exists $h->{sheet};
 
-  my $csvpath = $h->{outpath}; # might be same as {inpath} if already a CSV
+  my $csvpath = $h->{outpath} // oops; # same as {inpath} if already a CSV
   open my $fh, "<", $csvpath or croak "$csvpath : $!\n";
   binmode $fh, ":crlf:encoding(".$h->{encoding}.")" or die "binmode:$!";
 
@@ -1446,7 +1654,7 @@ ENCODING may be a comma-separated list of encoding
 names which will be tried in the order until one seems to work
 (requires pre-reading the input file).
 If only one encoding is specified it will be used without trying it first.
-The default is "UTF-8,UTF-16BE,UTF-16LE,windows-1252".
+The default is "UTF-8,windows-1252".
 
 =item output_encoding => ENCODING
 
